import { FeatureStateModel } from '../features/models';
import { ProjectModel } from '../projects/models';
import { IntegrationModel } from './integrations/models';

export class EnvironmentAPIKeyModel {
    id: number;
    key: string;
    createdAt: number;
    name: string;
    clientApiKey: string;
    expiresAt?: number;
    active = true;

    constructor(
        id: number,
        key: string,
        createdAt: number,
        name: string,
        clientApiKey: string,
        expiresAt?: number
    ) {
        this.id = id;
        this.key = key;
        this.createdAt = createdAt;
        this.name = name;
        this.clientApiKey = clientApiKey;
        this.expiresAt = expiresAt;
    }

    isValid() {
        return !!this.active && (!this.expiresAt || this.expiresAt > Date.now());
    }
}

export class EnvironmentModel {
    id: number;
    apiKey: string;
    project: ProjectModel;
    featureStates: FeatureStateModel[] = [];
    amplitude_config?: IntegrationModel;
    segment_config?: IntegrationModel;
    mixpanel_config?: IntegrationModel;
    heap_config?: IntegrationModel;

    constructor(id: number, apiKey: string, project: ProjectModel) {
        this.id = id;
        this.apiKey = apiKey;
        this.project = project;
    }
}
